#include <Servo.h>

enum {SERVO_RH=0, SERVO_RK, SERVO_RA, SERVO_LH, SERVO_LK, SERVO_LA};

Servo aServos[6];
int   aiLastPositions[6] = {-1, -1, -1, -1, -1};
bool runOnce=false;

// First pass Timed move.  Currently only handles 1 servo and
// still in degrees, would be better in either parts of degrees or
// convert to microseconds...
void ServoTimedMove(int iServo, int NewPosition, int Time) {
  // can not do a timed move for ones we don't know their current position
  if ((Time != 0) && (aiLastPositions[iServo] != -1)) {
    int Delta = NewPosition - aiLastPositions[iServo];
    int CycleTime = 20;
    int CntOfCycles = Time / 20;  // 20ms is natural recycle time for servos
    int DeltaPerCycle = Delta/CntOfCycles;
    if (DeltaPerCycle  == 0) {
      // Not enough change to do it so...
      DeltaPerCycle = (Delta > 0)?1 : -1;  // assume a change per cycle.
      CntOfCycles = abs(Delta);
      CycleTime = Time/CntOfCycles;
    }
    int iPos = aiLastPositions[iServo];
    while (CntOfCycles--) {
      iPos += DeltaPerCycle;
      aServos[iServo].write(iPos);
      delay(CycleTime);
    }
  }
  aServos[iServo].write(NewPosition);
  aiLastPositions[iServo] = NewPosition;
}

void standStraight () {
  // Stand straight
  ServoTimedMove(SERVO_RH, 100, 0);
  ServoTimedMove(SERVO_RK, 100, 0);
  ServoTimedMove(SERVO_RA, 85, 0);

  ServoTimedMove(SERVO_LH, 100, 0);
  ServoTimedMove(SERVO_LK, 100, 0);
  ServoTimedMove(SERVO_LA, 120, 0);
}

void kickBall () {
  // Lean the BRAT to it's right side
  ServoTimedMove(SERVO_RA, 130, 500);
  ServoTimedMove(SERVO_LA, 145, 1000); 

  // Swing the left leg back
  ServoTimedMove(SERVO_RH, 188, 500);

  // Straighten the left foot
  ServoTimedMove(SERVO_RA, 90, 500);

  // Swing left leg to kick
  ServoTimedMove(SERVO_RH, 50, 500);

  // Put left leg down
  ServoTimedMove(SERVO_RH, 100, 500);
}

void detachServos() {
  aServos[SERVO_LH].detach();  // servo Left hip is on port 4
  aServos[SERVO_LK].detach();  // servo Left knee is on port 3
  aServos[SERVO_LA].detach();  // servo Left ankle is on port 2

  aServos[SERVO_RH].detach();  // servo Right hip is on port 10
  aServos[SERVO_RK].detach();  // servo Right knee is on port 4
  aServos[SERVO_RA].detach();  // servo Right ankle is on port 4
}

void setup()
{ 
  //this declares what servo is on what port
  aServos[SERVO_LH].attach(4);  // servo Left hip is on port 4
  aServos[SERVO_LK].attach(3);  // servo Left knee is on port 3
  aServos[SERVO_LA].attach(2);  // servo Left ankle is on port 2

  aServos[SERVO_RH].attach(12);  // servo Right hip is on port 10
  aServos[SERVO_RK].attach(11);  // servo Right knee is on port 4
  aServos[SERVO_RA].attach(10);  // servo Right ankle is on port 4
}

void loop()
{
  if (!runOnce) {
    standStraight();
    delay(1000);
    kickBall();
    standStraight();
    delay (2000);
    detachServos();
    runOnce=1;  // Set to true so it does not run again.
  }
  }

